SystemOrganization addCategory: #'Interaction-Actions'!
SystemOrganization addCategory: #'Interaction-Actions-Morphic'!
SystemOrganization addCategory: #'Interaction-Actions-Tests'!

!MenuIcons class methodsFor: '*Interaction-Actions-Morphic' stamp: 'BEL 11/1/2005 23:28'!
decorateMenu: aMenu 
	"decorate aMenu with icons"
	| numberAdded |
	Preferences menuWithIcons
		ifFalse: [^ self].
	aMenu ifNil: [^self].
	numberAdded := 0.
	aMenu items
		do: [:item | 
			| icon | 
			icon := self iconForMenuItem: item.
			icon
				ifNotNil: [item icon: icon.
					numberAdded := numberAdded + 1]].
	numberAdded isZero
		ifTrue: [^ self].
	aMenu addBlankIconsIfNecessary: self blankIcon! !

TestCase subclass: #GenericActionTest
	instanceVariableNames: 'clubName rank atWar'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Interaction-Actions-Tests'!

!GenericActionTest commentStamp: 'BEL 12/3/2006 15:18' prior: 0!
GenericActionTest

These tests test GenericAction and its subclasses.  The text in the examples comes from Bill Watterson's Calvin & Hobbes cartoons, because I had a book of them handy.


Copyright (c) 2005-2006 Brenda Larcom <asparagi@hhhh.org>

Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

!

!GenericActionTest methodsFor: 'actions' stamp: 'BEL 10/24/2005 00:48'!
allActions
	"Answer an OrderedCollection of GenericActions."
	| result |
	result := OrderedCollection new.
	result
		add: (self genericAction: 'Calvin''s a Dope');
		 add: self editableAction;
		 add: self toggleAction.
	^ result! !

!GenericActionTest methodsFor: 'actions' stamp: 'BEL 10/24/2005 00:40'!
allActionsAction
	"Answer an action which will answer all actions."
	^ GenericActions
		label: 'all actions'
		receiver: self
		selector: #allActions! !

!GenericActionTest methodsFor: 'accessing' stamp: 'BEL 10/24/2005 00:32'!
atWar
	"Answer whether Calvin & Hobbes are at war."
	^atWar! !

!GenericActionTest methodsFor: 'accessing' stamp: 'BEL 11/1/2005 21:58'!
atWar: aBoolean 
	"Set whether Calvin & Hobbes are at war."
	atWar := aBoolean.
	self triggerEvent: #atWar! !

!GenericActionTest methodsFor: 'accessing' stamp: 'BEL 10/23/2005 23:53'!
clubName
	"Answer the name of Calvin & Hobbes' club."
	^ clubName! !

!GenericActionTest methodsFor: 'accessing' stamp: 'BEL 11/1/2005 21:58'!
clubName: aString 
	"Set the name of Calvin & Hobbes' club."
	clubName := aString.
	self triggerEvent: #clubName! !

!GenericActionTest methodsFor: 'actions' stamp: 'BEL 10/24/2005 00:49'!
editableAction
	"Answer an action in which Hobbes sings the club anthem  
	early."
	| action |
	action := EditableAction
				label: ''
				receiver: self
				selector: #singAnthem.
	action getLabelSelector: #clubName setLabelSelector: #clubName:.
	^ action! !

!GenericActionTest methodsFor: 'actions' stamp: 'BEL 10/24/2005 00:48'!
genericAction: aString 
	"Answer an action to set the name of Calvin & Hobbes' club to 
	the given string."
	^ GenericAction
		label: aString
		receiver: self
		selector: #clubName:
		arguments: {aString}! !

!GenericActionTest methodsFor: 'accessing' stamp: 'BEL 10/23/2005 23:53'!
rank
	"Answer Calvin's rank in Calvin & Hobbes' club."
	^ rank! !

!GenericActionTest methodsFor: 'accessing' stamp: 'BEL 11/1/2005 21:58'!
rank: aString 
	"Set Calvin's rank in Calvin & Hobbes' club."
	rank := aString.
	self triggerEvent: #rank! !

!GenericActionTest methodsFor: 'set up' stamp: 'BEL 10/24/2005 00:31'!
setUp
	"Prepare my test fixture."
	super setUp.
	clubName := 'G.R.O.S.S.'.
	rank := 'Dictator For Life'.
	atWar := false! !

!GenericActionTest methodsFor: 'actions' stamp: 'BEL 10/24/2005 00:38'!
singAnthem
	"Simulate Hobbes singing the club anthem."
	self clubName: 'Hobbes is a mangy flea-ridden furball';
		rank: 'Club Chowder Head';
		atWar: true! !

!GenericActionTest methodsFor: 'triggering' stamp: 'BEL 10/24/2005 00:49'!
testEditableAction
	| action |
	action := self editableAction.
	self
		should: [action label = 'G.R.O.S.S.'];
		
		shouldnt: [action answersActions];
		
		should: [action hasEditableLabel];
		
		shouldnt: [action hasState].
	action trigger.
	self
		should: [action label = 'Hobbes is a mangy flea-ridden furball']! !

!GenericActionTest methodsFor: 'triggering' stamp: 'BEL 10/24/2005 00:48'!
testGenericAction
	| label action |
	label := 'Get Rid Of Slimy girlS'.
	action := self genericAction: label.
	self
		should: [action label = label];
		
		shouldnt: [action answersActions];
		
		shouldnt: [action hasEditableLabel];
		
		shouldnt: [action hasState];
		
		should: [self clubName = 'G.R.O.S.S.'].
	action trigger.
	self
		should: [self clubName = label]! !

!GenericActionTest methodsFor: 'triggering' stamp: 'BEL 10/24/2005 00:50'!
testGenericActions
	| action result |
	action := self allActionsAction.
	self
		should: [action answersActions];
		
		shouldnt: [action hasEditableLabel];
		
		shouldnt: [action hasState].
	result := action value.
	self should: [result size = 3]! !

!GenericActionTest methodsFor: 'triggering' stamp: 'BEL 10/24/2005 00:48'!
testToggleAction
	| action |
	action := self toggleAction.
	self
		shouldnt: [action answersActions];
		
		shouldnt: [action hasEditableLabel];
		
		should: [action hasState];
		
		shouldnt: [self atWar].
	action trigger.
	self
		should: [self atWar].
	action trigger.
	self
		shouldnt: [self atWar]! !

!GenericActionTest methodsFor: 'actions' stamp: 'BEL 10/24/2005 00:47'!
toggleAction
	"Answer an action in which Calvin & Hobbes toggle their atWar 
	state."
	| action |
	action := ToggleAction
				label: 'at war'
				receiver: self
				selector: #toggleAtWar.
	action getStateSelector: #atWar.
	^ action! !

!GenericActionTest methodsFor: 'actions' stamp: 'BEL 11/1/2005 22:05'!
toggleAtWar
	"Toggle whether Calvin & Hobbes are at war."
	self atWar: atWar not! !

GenericActionTest subclass: #MorphicInterfaceActionTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Interaction-Actions-Tests'!

!MorphicInterfaceActionTest commentStamp: 'BEL 12/3/2006 15:18' prior: 0!
MorphicInterfaceAction

These tests test the basic Morphic subclasses of HumanInterfaceAction.  The examples are copied from GenericActionTest.


Copyright (c) 2005-2006 Brenda Larcom <asparagi@hhhh.org>

Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.!

!MorphicInterfaceActionTest methodsFor: 'interface' stamp: 'BEL 11/1/2005 22:56'!
testEditableButton
	| target interfaceAction interface |
	target := self editableAction.
	interfaceAction := MorphicButtonAction wrap: target.
	interface := interfaceAction control.
	self
		should: [interfaceAction label = target label];
		
		should: [interface model = target].
	target trigger.
	self
		should: [interfaceAction label = target label]! !

!MorphicInterfaceActionTest methodsFor: 'interface' stamp: 'BEL 11/1/2005 23:34'!
testGenericActionsButton
	| interfaceAction interface |
	interfaceAction := MorphicButtonAction
				wrap: (self allActionsAction).
	interface := interfaceAction control.
	self
		should: [interface isNil]! !

!MorphicInterfaceActionTest methodsFor: 'interface' stamp: 'BEL 12/3/2006 16:44'!
testGenericActionsMenu
	| target interfaceAction interface |
	target := self allActionsAction.
	interfaceAction := MorphicMenuAction wrap: target.
	interface := interfaceAction control.
	self
		should: [interfaceAction label = target label];
		
		should: [interface defaultTarget = nil];
		
		should: [target value size = interface items size]! !

!MorphicInterfaceActionTest methodsFor: 'interface' stamp: 'BEL 12/3/2006 16:39'!
testGenericActionsMenuItem
	| target interfaceAction interface |
	target := self allActionsAction.
	interfaceAction := MorphicMenuItemAction wrap: target.
	interface := interfaceAction control.
	self
		should: [interfaceAction label = target label];
		
		should: [target label = interface contents];
		
		should: [interface hasSubMenu];
		
		should: [target value size = interface subMenu items size]! !

!MorphicInterfaceActionTest methodsFor: 'interface' stamp: 'BEL 11/1/2005 22:36'!
testGenericButton
	| label target interfaceAction interface |
	label := 'Get Rid Of Slimy girlS'.
	target := self genericAction: label.
	interfaceAction := MorphicButtonAction wrap: target.
	interface := interfaceAction control.
	self
		should: [interfaceAction label = target label];
		should: [interface model = target].! !

!MorphicInterfaceActionTest methodsFor: 'interface' stamp: 'BEL 12/3/2006 16:39'!
testGenericMenuItem
	| label target interfaceAction interface |
	label := 'Get Rid Of Slimy girlS'.
	target := self genericAction: label.
	interfaceAction := MorphicMenuItemAction wrap: target.
	interface := interfaceAction control.
	self
		should: [interfaceAction label = target label];
		
		should: [interface target = target]! !

!MorphicInterfaceActionTest methodsFor: 'interface' stamp: 'BEL 12/3/2006 16:39'!
testGenericMenuItemWithKeystroke
	| label target interfaceAction interface |
	label := 'Get Rid Of Slimy girlS'.
	target := self genericAction: label.
	interfaceAction := MorphicMenuItemAction wrap: target.
	interfaceAction keystroke: 'k'.
	interface := interfaceAction control.
	self
		shouldnt: [interfaceAction label = target label];
		
		should: [interface target = target]! !

!MorphicInterfaceActionTest methodsFor: 'interface' stamp: 'BEL 11/1/2005 21:31'!
testGenericPreference
	| interfaceAction label interface |
	label := 'Get Rid Of Slimy girlS'.
	interfaceAction := MorphicPreferenceAction
				wrap: (self genericAction: label).
	interface := interfaceAction control.
	self should: [interface isNil]! !

!MorphicInterfaceActionTest methodsFor: 'interface' stamp: 'BEL 11/1/2005 22:46'!
testToggleButton
	| target interfaceAction interface |
	target := self toggleAction.
	interfaceAction := MorphicButtonAction wrap: target.
	interface := interfaceAction control.
	self 
		should: [interfaceAction label = target label];
		should: [interface model = target];	
		shouldnt: [interface isOn].
	target trigger.
	self
		should: [interface isOn]! !

!MorphicInterfaceActionTest methodsFor: 'interface' stamp: 'BEL 12/3/2006 16:36'!
testToggleMenuItem
	| target interfaceAction interface |
	target := self toggleAction.
	interfaceAction := MorphicMenuItemAction wrap: target.
	interface := interfaceAction control.
	self
		shouldnt: [interfaceAction label = target label];
		
		should: [interface hasMarker];
		
		should: [interface target = target];
		
		should: [interface contentString = interfaceAction label].
	target trigger.
	self
		should: [interface contentString = interfaceAction label]! !

!MorphicInterfaceActionTest methodsFor: 'interface' stamp: 'BEL 11/1/2005 21:45'!
testTogglePreference
	| target interfaceAction interface checkbox |
	target := self toggleAction.
	interfaceAction := MorphicPreferenceAction wrap: target.
	interface := interfaceAction control.
	checkbox := interface submorphs first.
	self
		should: [interfaceAction label = target label];
		should: [interface submorphs size = 3];
		shouldnt: [checkbox isOn].
	target trigger.
	self should: [checkbox isOn].! !

Object subclass: #HumanInterfaceAction
	instanceVariableNames: 'genericAction'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Interaction-Actions'!

!HumanInterfaceAction commentStamp: 'BEL 12/3/2006 14:53' prior: 0!
HumanInterfaceAction

I am an abstract class.  Instances of my concrete subclasses can create and configure human interface elements which enable humans to trigger specific GenericActions.  Therefore, my instances contain a GenericAction and presentation information specific to a particular human sense (e.g. sight, hearing) and interface framework.   My concrete subclasses typically contain a stash of presentation information which they draw on when initializing their instances.

Instance variables:

genericAction		the GenericAction I embellish


Copyright (c) 2005-2006 Brenda Larcom <asparagi@hhhh.org>

Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.!

!HumanInterfaceAction class methodsFor: 'instance creation' stamp: 'BEL 10/19/2005 12:41'!
wrapAll: genericActionCollection
	"Answer a collection of new instances of me, initialized with the GenericActions in the given collection."
	^genericActionCollection collect: [:action |
		self wrap: action]! !

!HumanInterfaceAction class methodsFor: 'instance creation' stamp: 'BEL 11/27/2005 14:28'!
wrap: genericAction 
	"Answer a new instance of me, initialized from the given 
	GenericAction. "
	^ genericAction ifNotNil: [self new initializeFrom: genericAction]! !

!HumanInterfaceAction methodsFor: 'interface' stamp: 'BEL 10/19/2005 14:48'!
control
	"Answer an appropriately configured human interface element for triggering my genericAction."
	
	self subclassResponsibility! !

!HumanInterfaceAction methodsFor: 'initialization' stamp: 'BEL 10/19/2005 14:46'!
initializeFrom: anAction
	"Set my initial state."

	genericAction := anAction! !

!HumanInterfaceAction methodsFor: 'accessing' stamp: 'BEL 11/1/2005 21:22'!
label
	"Answer my label."
	
	^genericAction label! !

HumanInterfaceAction subclass: #MorphicButtonAction
	instanceVariableNames: 'icon'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Interaction-Actions-Morphic'!

!MorphicButtonAction commentStamp: 'BEL 12/3/2006 15:43' prior: 0!
MorphicButtonAction

Instances of me create & configure button Morphs to allow a human to trigger a GenericAction.  I handle editable labels & states.  I also have minimal support for icons, but the interface builder must manually configure them.   Subclassing me for application-specific icon support is recommended. 

Instance variables:

icon		the Icon to display on buttons I create

Copyright (c) 2005-2006 Brenda Larcom <asparagi@hhhh.org>

Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.!

!MorphicButtonAction methodsFor: 'interface' stamp: 'BEL 12/14/2005 19:26'!
control
	"Answer a Morphic button for triggering my genericAction."
	| stateSelector result labelSelector |
	genericAction answersActions ifTrue: [^nil].
	stateSelector := genericAction hasState
				ifTrue: [#state].
	labelSelector := genericAction hasEditableLabel
				ifTrue: [#label].
	result := PluggableButtonMorphPlus
				on: genericAction
				getState: stateSelector
				action: #trigger
				label: labelSelector
				menu: nil.
	result beTransparent; borderWidth: 2; borderRaised; hResizing: #shrinkWrap; vResizing: #spaceFill; label: self labelMorph; setBalloonText: genericAction description.
	stateSelector
		ifNotNil: [genericAction
				when: stateSelector
				send: #update:
				to: result
				with: stateSelector].
	labelSelector
		ifNotNil: [genericAction
				when: labelSelector
				send: #update:
				to: result
				with: labelSelector].
	^ result! !

!MorphicButtonAction methodsFor: 'accessing' stamp: 'BEL 12/3/2006 15:34'!
icon
	"Answer my icon."

	^icon! !

!MorphicButtonAction methodsFor: 'accessing' stamp: 'BEL 12/3/2006 15:34'!
icon: anIcon
	"Set my icon."

	icon := anIcon! !

!MorphicButtonAction methodsFor: 'interface' stamp: 'BEL 10/24/2005 20:01'!
labelMorph
	"Answer a Morph to use as my label."
	| iconToUse |
	iconToUse := self icon.
	^ iconToUse
		ifNil: [StringMorph contents: self label font: Preferences standardButtonFont]
		ifNotNil: [SketchMorph withForm: iconToUse]! !

HumanInterfaceAction subclass: #MorphicMenuAction
	instanceVariableNames: 'icon keystroke'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Interaction-Actions-Morphic'!

!MorphicMenuAction commentStamp: 'BEL 12/3/2006 16:30' prior: 0!
MorphicMenuAction

Instances of me create & configure a menu Morph as needed to allow a human to trigger the items in a GenericActions.   I use MorphicMenuItemAction.


Copyright (c) 2005-2006 Brenda Larcom <asparagi@hhhh.org>

Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.!

!MorphicMenuAction methodsFor: 'interface' stamp: 'BEL 12/3/2006 16:35'!
control
	"Answer a menu with items for each of my genericAction's 
	values. This will cause a lot of trouble if genericAction 
	answersActions is false."
	| result |
	result := MenuMorph new.
	(MorphicMenuItemAction wrapAll: genericAction value)
		do: [:interfaceAction | result addMorphBack: interfaceAction control].
	result addBlankIconsIfNecessary: MenuIcons blankIcon.
	result beSticky.
	^ result! !

HumanInterfaceAction subclass: #MorphicMenuItemAction
	instanceVariableNames: 'icon keystroke'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Interaction-Actions-Morphic'!

!MorphicMenuItemAction commentStamp: 'BEL 12/3/2006 16:38' prior: 0!
MorphicMenuItemAction

Instances of me create & configure a menu item Morph as needed to allow a human to trigger a GenericAction.  I handle states and submenus (using MorphicMenuAction if my genericAction is actually a GenericActions).  I also have minimal support for icons & keystrokes, but the interface builder must manually configure them.   Subclassing me for application-specific icon & keystroke support is recommended. 

In the future it would be nice not to have to build all the submenus that will probably not be used.

Instance variables:

icon		the small Icon to display on the left of the menu I create
keystroke	the character a human could press to quickly navigate to the menu item I create


Copyright (c) 2005-2006 Brenda Larcom <asparagi@hhhh.org>

Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.!

!MorphicMenuItemAction methodsFor: 'interface' stamp: 'BEL 12/3/2006 16:21'!
control
	"Answer a menu item for invoking me."
	| result |
	result := (genericAction hasState
				ifTrue: [UpdatingMenuItemMorph]
				ifFalse: [MenuItemMorph]) new.
	result contents: self label;
		 setBalloonText: genericAction description.
	genericAction answersActions
		ifTrue: [result subMenu: (MorphicMenuAction wrap: genericAction) control]
		ifFalse: [result target: genericAction;
				selector: #trigger;
				arguments: #()].
	genericAction hasState
		ifTrue: [result wordingProvider: self wordingSelector: #label.
			genericAction
				when: #state
				send: #updateContents
				to: result].
	Preferences menuWithIcons & self icon notNil
		ifTrue: [result icon: self icon].
	^ result! !

!MorphicMenuItemAction methodsFor: 'accessing' stamp: 'BEL 12/3/2006 15:59'!
icon
	"Answer my icon."

	^icon! !

!MorphicMenuItemAction methodsFor: 'accessing' stamp: 'BEL 12/3/2006 15:59'!
icon: anIcon
	"Set my icon."

	icon := anIcon! !

!MorphicMenuItemAction methodsFor: 'accessing' stamp: 'BEL 12/3/2006 15:59'!
keystroke
	"Answer the keystroke that would invoke my genericAction."

	^keystroke! !

!MorphicMenuItemAction methodsFor: 'accessing' stamp: 'BEL 12/3/2006 15:59'!
keystroke: aCharacter
	"Set the keystroke that would invoke my genericAction."

	keystroke := aCharacter! !

!MorphicMenuItemAction methodsFor: 'accessing' stamp: 'BEL 12/3/2006 15:59'!
label
	| result |
	result := genericAction label.
	genericAction hasState
		ifTrue: [| currentState | 
			currentState := genericAction state.
			currentState
				ifNotNil: [result := (currentState
								ifTrue: ['<yes>']
								ifFalse: ['<no>'])
								, result]].
	keystroke
		ifNotNil: [result := result , ' (' , keystroke asString , ')'].
	^ result! !

HumanInterfaceAction subclass: #MorphicPreferenceAction
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Interaction-Actions-Morphic'!

!MorphicPreferenceAction commentStamp: 'BEL 12/3/2006 15:26' prior: 0!
MorphicPreferenceAction

Instances of me create & configure checkbox & label Morphs to allow a human to trigger a ToggleAction.  I should probably be refactored to match (and possibly replace) PreferenceView and its subclasses; currently I am roughly equivalent to BooleanPreferenceView.


Copyright (c) 2005-2006 Brenda Larcom <asparagi@hhhh.org>

Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.!

!MorphicPreferenceAction methodsFor: 'interface' stamp: 'BEL 11/1/2005 22:29'!
control
	"Answer a small Morph for changing my value, or nil if no such 
	element is appropriate."
	| outerButton aButton str |
	genericAction hasState
		ifFalse: [^ nil].
	aButton := UpdatingThreePhaseButtonMorph checkBox.
	aButton target: genericAction;
		 actionSelector: #trigger;
		 getSelector: #state.
	genericAction
		when: #state
		send: #step
		to: aButton.
	str := StringMorph contents: self label.
	str lock.
	outerButton := AlignmentMorph newRow height: 24.
	outerButton beTransparent; beSticky; hResizing: #spaceFill; vResizing: #shrinkWrap; setNameTo: self label; addMorph: aButton; addTransparentSpacerOfSize: 2 @ 0; addMorphBack: str;
		on: #mouseUp
		send: #trigger
		to: genericAction; setBalloonText: genericAction description; setProperty: #balloonTarget toValue: aButton.
	^ outerButton! !

MessageSend subclass: #GenericAction
	instanceVariableNames: 'label description'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Interaction-Actions'!

!GenericAction commentStamp: 'BEL 12/3/2006 15:07' prior: 0!
Generic Action Framework

This simple framework defines an abstraction layer between a model and a human interface.   Using this abstraction, the model answers GenericActions (slightly extended MessageSends) a human might want to perform, and the human interface uses appropriate subclasses of HumanInterfaceAction to construct a human interface to these GenericActions.

Advantages:

- As the model's behavior is extended, interfaces to the model can automatically extend themselves. 
- The model need not be extended to support a new interface framework, or different uses of the same interface framework.
- Much similar code for creating and configuring human interface elements can be centralized in subclasses of HumanInterfaceAction and therefore eliminated from the collaborating human interface.  
- Because of this centralization of similar element creation code, it will be easy for a programmer to create human interfaces with predictable behavior.
- Because of the combination of centralized element creation code and lack of interface knowledge in the model, it will be easier for the community to adopt new interface frameworks.

Usage:

Creation of a human interface to some model object typically begins with the creation of the largest containing interface element in some particular interface framework.  This top element creates its contained interface elements when it is initialized with the model, and they in turn create their contained elements when they are initialized. 

When a containing element wants to create contained controls, it sends #interestingActions to its model (or sometimes other interface elements, to create controls for those elements), which responds with a Collection of GenericActions objects which represent categories of GenericActions.  The containing element may immediately trigger some of the GenericActions objects to get more GenericActions. 

When the containing element has the GenericActions it wants controls for, it wraps them in instances of a concrete subclass of HumanInterfaceAction, typically by calling subclass>>#wrapAll: on the Collection.  When each HumanInterfaceAction is initialized from a GenericAction, it adds the additional information needed to configure a control from the interface framework it uses for this action on this model.  Typically this information is stored in the HumanInterfaceAction subclass.

Now the containing element can send #control to ask each HumanInterfaceAction to create and configure a control which triggers its GenericAction.  The containing element adds this control to itself.

Known rough spots (framework may be redesigned in the future to fix these issues):

- If only some GenericActions categories are triggered immediately to build the collection of GenericActions which will be available through the human interface, the human interface must understand at least a little about the GenericAction categories that will be offered by the model in order to choose which to trigger immediately.  This coupling of interface to model is not desirable, but may be inevitable.
- Because of the above, superclass authors must think carefully about the needs of possible future subclasses when they design categories of GenericActions. 
- If a human interface element requires application-specific configuration which is not inherent in the model (e.g. colors which correspond to values in the model), the model cannot supply the configuration because it should not know about the human interface.  Therefore, the human interface must again understand the model enough to do this configuration.   One (untried) solution might be to make application-specific subclasses of HumanInterfaceAction subclasses.

Credits:

This framework was inspired by OBAction, from the OmniBrowser package.


GenericAction

I am derived from MessageSend.  My instances represent messages a human might want to send.   They contain generic information a human could use to infer what would happen if she sent this message.  They do not contain information or behavior specific to any particular sensory mode (e.g. no sounds, images, et cetera) or human interface framework.

My instances are typically wrapped by instances of HumanInterfaceAction's concrete subclasses, and triggered when a human invokes a control created by the wrapping instance.

Instance variables:

label			a String containing a human-comprehensible name for me
description	a String containing a human-readable description of my effects


Copyright (c) 2005-2006 Brenda Larcom <asparagi@hhhh.org>

Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.!

GenericAction subclass: #EditableAction
	instanceVariableNames: 'labelReceiver getLabelSelector setLabelSelector labelArguments'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Interaction-Actions'!

!EditableAction commentStamp: 'BEL 12/3/2006 15:17' prior: 0!
EditableAction

My instances represent messages whose label can be edited by a human.  Typically instances of me would be used for messages which are somehow proxy-like, e.g. my use in SpiralNotebook to change which section is visible.  In this case, labelReceiver is the object being proxied and receiver is the object which will manipulate a proxy for labelReceiver.

Instance variables:

labelReceiver			an Object which will receive label-related messages
getLabelSelector		a Symbol containing the name of a method which, when sent to my labelReceiver, will answer my label
setLabelSelector		a Symbol containing the name of a method which, when sent to my labelReceiver, will set my label
labelArguments		an Array of arguments for getLabelSelector and setLabelSelector; for setLabelSelector the new label will be added as an additional  argument at the end of the array


Copyright (c) 2005-2006 Brenda Larcom <asparagi@hhhh.org>

Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.!

!EditableAction methodsFor: 'accessing' stamp: 'BEL 10/19/2005 14:31'!
getLabelSelector: aSelector setLabelSelector: oneMoreArgumentSelector
	"Set the selectors for getting & setting my label."
	
	self getLabelSelector:  aSelector setLabelSelector: oneMoreArgumentSelector arguments: nil! !

!EditableAction methodsFor: 'accessing' stamp: 'BEL 10/19/2005 14:41'!
getLabelSelector: aSelector setLabelSelector: oneMoreArgumentSelector arguments: anArray
	"Set my selectors and arguments for getting & setting my label.   The new labelArguments array should contain the correct number of arguments for getLabelSelector, and one too few arguments for setLabelSelector; when setLabelSelector is sent, the new label string will be added to labelArguments following the last element."
	
	self unregisterForEvents.
	getLabelSelector := aSelector.
	setLabelSelector := oneMoreArgumentSelector.
	labelArguments := anArray.
	self registerForEvents! !

!EditableAction methodsFor: 'accessing' stamp: 'BEL 11/2/2005 11:47'!
hasEditableLabel
	"Answers whether my label is editable."
	^ self labelReceiver notNil
		and: [setLabelSelector notNil]! !

!EditableAction methodsFor: 'accessing' stamp: 'BEL 11/2/2005 11:48'!
label
	"Answer my label String."

	^(self labelReceiver isNil or: [getLabelSelector isNil]) 
		ifTrue: [super label]
		ifFalse: 
			[labelArguments ifNil: [self labelReceiver perform: getLabelSelector]
				ifNotNil: [self labelReceiver perform: getLabelSelector withArguments: labelArguments]]! !

!EditableAction methodsFor: 'accessing' stamp: 'BEL 11/2/2005 11:46'!
labelReceiver
	"Answer the object which should receive getLabelSelector and setLabelSelector."
	^labelReceiver ifNil: [receiver]! !

!EditableAction methodsFor: 'accessing' stamp: 'BEL 11/2/2005 11:50'!
labelReceiver: anObject 
	"Set the receiver of getLabelSelector and setLabelSelector to 
	anObject."
	self unregisterForEvents.
	labelReceiver := anObject.
	self registerForEvents! !

!EditableAction methodsFor: 'accessing' stamp: 'BEL 11/2/2005 11:48'!
label: aString 
	"Set my label String. I rely on my model to trigger a 
	getLabelSelector event when this change occurs."

	(self labelReceiver isNil or: [setLabelSelector isNil]) 
		ifTrue: [super label: aString]
		ifFalse: 
			[labelArguments
				ifNil: [self labelReceiver perform: setLabelSelector with: aString]
				ifNotNil: 
					[self labelReceiver perform: setLabelSelector
						withArguments: labelArguments , { 
										aString}]]! !

!EditableAction methodsFor: 'event handling' stamp: 'BEL 11/11/2005 08:44'!
registerForEvents
	"Register for a getLabelSelector event from my receiver."

	(self labelReceiver notNil and: [getLabelSelector notNil]) 
		ifTrue: 
			[self labelReceiver 
				when: getLabelSelector
				send: #triggerEvent:
				to: self
				with: #label]! !

!EditableAction methodsFor: 'event handling' stamp: 'BEL 11/2/2005 11:49'!
unregisterForEvents
	"Unregister for a getLabelSelector event from my receiver."

	(self labelReceiver notNil and: [getLabelSelector notNil]) 
		ifTrue: [self labelReceiver removeActionsWithReceiver: self forEvent: getLabelSelector]! !

GenericAction subclass: #GenericActions
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Interaction-Actions'!

!GenericActions commentStamp: 'BEL 12/3/2006 15:17' prior: 0!
GenericActions

My instances represent messages which answer Collections of other GenericAction objects.  Therefore, my selector must answer a Collection or nil.  Typically, a model uses instances of me to categorize GenericAction objects.


Copyright (c) 2005-2006 Brenda Larcom <asparagi@hhhh.org>

Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.!

!GenericActions methodsFor: 'accessing' stamp: 'BEL 10/19/2005 14:53'!
answersActions
	"Answers whether >>value answers a collection of GenericActions."
	
	^receiver notNil and: [selector notNil]! !

!GenericAction class methodsFor: 'instance creation' stamp: 'BEL 10/20/2005 17:04'!
label: aString receiver: anObject selector: aSelector
	^self label: aString receiver: anObject selector: aSelector arguments: #()! !

!GenericAction class methodsFor: 'instance creation' stamp: 'BEL 10/19/2005 13:19'!
label: aString receiver: anObject selector: aSelector arguments: anArray 
	^(self receiver: anObject selector: aSelector arguments: anArray) label: aString! !

!GenericAction methodsFor: 'accessing' stamp: 'BEL 10/19/2005 14:53'!
answersActions
	"Answers whether >>value answers a collection of GenericActions."
	
	^false

	! !

!GenericAction methodsFor: 'accessing' stamp: 'BEL 10/19/2005 13:17'!
description
	"Answer my description String."
	
	^description! !

!GenericAction methodsFor: 'accessing' stamp: 'BEL 10/19/2005 13:17'!
description: aString
	"Set my description String.  This String should be a meaningful (to humans) description of the effects of the message I represent."
	
	description := aString! !

!GenericAction methodsFor: 'accessing' stamp: 'BEL 10/19/2005 14:50'!
hasEditableLabel
	"Answers whether my label is editable."
	
	^false

	! !

!GenericAction methodsFor: 'accessing' stamp: 'BEL 10/19/2005 14:49'!
hasState
	"Answers whether I have a Boolean state."
	
	^false

	! !

!GenericAction methodsFor: 'accessing' stamp: 'BEL 10/19/2005 13:17'!
label
	"Answer my label String."
	
	^label! !

!GenericAction methodsFor: 'accessing' stamp: 'BEL 10/19/2005 13:16'!
label: aString
	"Set my label String.  This String should be a meaningful (to humans) but short name for the message I represent."
	
	label := aString! !

!GenericAction methodsFor: 'event handling' stamp: 'BEL 11/28/2005 18:36'!
perform: selectorSymbol orSendTo: otherTarget 
	"This unnecessary complexity is here because of 
	MenuMorph>>setInvokingView:, which rewrites all menu 
	items with no arguments to call this.  Isn't it fun to layer crufty code on top of crufty code?"
	^ self perform: selectorSymbol! !

!GenericAction methodsFor: 'accessing' stamp: 'BEL 10/19/2005 14:42'!
receiver: anObject
	"Set my receiver."
	
	self unregisterForEvents.
	super receiver: anObject.
	self registerForEvents! !

!GenericAction methodsFor: 'event handling' stamp: 'BEL 10/19/2005 14:41'!
registerForEvents
	"Register for events from my receiver."! !

!GenericAction methodsFor: 'event handling' stamp: 'BEL 10/19/2005 14:53'!
trigger
	"Invoke myself."
	
	self value! !

!GenericAction methodsFor: 'event handling' stamp: 'BEL 10/19/2005 14:41'!
unregisterForEvents
	"Unregister for events from my receiver."! !

GenericAction subclass: #ToggleAction
	instanceVariableNames: 'getStateSelector getStateArguments'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Interaction-Actions'!

!ToggleAction commentStamp: 'BEL 12/3/2006 15:17' prior: 0!
ToggleAction

My instances represent messages which toggle a boolean state.   For each of my instances, selector should alter the state answered by getStateSelector.

Instance variables:

getStateSelector		a Symbol containing the name of a method which, when sent to my receiver, will answer true or false
getStateArguments	an Array of arguments for getStateSelector


Copyright (c) 2005-2006 Brenda Larcom <asparagi@hhhh.org>

Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.!

!ToggleAction methodsFor: 'accessing' stamp: 'BEL 10/19/2005 14:33'!
getStateSelector: aSymbol
	"Set the selector I use to retrieve the current state."

	self getStateSelector: aSymbol arguments: nil! !

!ToggleAction methodsFor: 'accessing' stamp: 'BEL 10/19/2005 14:43'!
getStateSelector: aSymbol arguments: argumentList
	"Set the selector and arguments I use to retrieve the current state."

	self unregisterForEvents.
	getStateSelector := aSymbol.
	getStateArguments := argumentList.
	self registerForEvents! !

!ToggleAction methodsFor: 'accessing' stamp: 'BEL 10/19/2005 14:51'!
hasState
	"Answers whether I have a Boolean state."
	
	^receiver notNil and: [getStateSelector notNil]! !

!ToggleAction methodsFor: 'event handling' stamp: 'BEL 10/19/2005 14:39'!
registerForEvents
	"Register for a getStateSelector event from my receiver."

	(receiver notNil and: [getStateSelector notNil]) ifTrue: [
		receiver when: getStateSelector send: #stateChanged to: self]! !

!ToggleAction methodsFor: 'accessing' stamp: 'BEL 10/19/2005 14:44'!
state
	"Answer my state."

	^getStateArguments
		ifNil: [receiver perform: getStateSelector]
		ifNotNil: [receiver perform: getStateSelector withArguments: getStateArguments]! !

!ToggleAction methodsFor: 'event handling' stamp: 'BEL 11/1/2005 22:29'!
stateChanged
	"My state has changed."
	self triggerEvent: #state! !

!ToggleAction methodsFor: 'event handling' stamp: 'BEL 10/19/2005 14:39'!
unregisterForEvents
	"Unregister for a getStateSelector event from my receiver."

	(receiver notNil and: [getStateSelector notNil]) ifTrue: [
		receiver removeActionsWithReceiver: self forEvent: getStateSelector]! !
